@@ -63,8 +63,7 @@
 
 typedef struct {
   enum {
-    PULL_MSG_SCAN_IDLE,
-    PULL_MSG_MAIN_IDLE,
+    PULL_MSG_IDLE,
     PULL_MSG_FETCH,
     PULL_MSG_FETCH_DETACHED_METADATA,
     PULL_MSG_SCAN,
@@ -87,6 +86,7 @@ typedef struct {
   GMainContext    *main_context;
   GMainLoop    *loop;
   GCancellable *cancellable;
+  OstreeAsyncProgress *progress;
 
   gboolean      transaction_resuming;
   volatile gint n_scanned_metadata;
@@ -102,8 +102,10 @@ typedef struct {
   GHashTable       *scanned_metadata; /* Maps object name to itself */
   GHashTable       *requested_metadata; /* Maps object name to itself */
   GHashTable       *requested_content; /* Maps object name to itself */
-  guint             metadata_scan_idle : 1; /* TRUE if we passed through an idle message */
-  guint             idle_serial; /* Incremented when we get a SCAN_IDLE message */
+  guint             checking_metadata_scan_complete : 1;
+  guint             metadata_scan_complete : 1;
+  gboolean          was_idle;
+  guint             idle_serial;
   guint             n_outstanding_metadata_fetches;
   guint             n_outstanding_metadata_write_requests;
   guint             n_outstanding_content_fetches;
@@ -179,47 +181,35 @@ suburi_new (SoupURI   *base,
 }
 
 static gboolean
-uri_fetch_update_status (gpointer user_data)
+update_progress (gpointer user_data)
 {
   OtPullData *pull_data = user_data;
-  GString *status;
-  guint outstanding_writes;
-  guint outstanding_fetches;
+  guint outstanding_writes = pull_data->n_outstanding_content_write_requests +
+    pull_data->n_outstanding_metadata_write_requests;
+  guint outstanding_fetches = pull_data->n_outstanding_content_fetches +
+    pull_data->n_outstanding_metadata_fetches;
+  guint64 bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
+  guint fetched = pull_data->n_fetched_metadata + pull_data->n_fetched_content;
+  guint requested = pull_data->n_requested_metadata + pull_data->n_requested_content;
+  guint n_scanned_metadata = g_atomic_int_get (&pull_data->n_scanned_metadata);
  
-  status = g_string_new ("");
+  g_assert (pull_data->progress);
 
-  outstanding_fetches = pull_data->n_outstanding_content_fetches + pull_data->n_outstanding_metadata_fetches;
-  outstanding_writes = pull_data->n_outstanding_content_write_requests + pull_data->n_outstanding_metadata_write_requests;
+  ostree_async_progress_set_uint (pull_data->progress, "outstanding-fetches", outstanding_fetches);
+  ostree_async_progress_set_uint (pull_data->progress, "outstanding-writes", outstanding_writes);
+  ostree_async_progress_set_uint (pull_data->progress, "fetched", fetched);
+  ostree_async_progress_set_uint (pull_data->progress, "requested", requested);
+  ostree_async_progress_set_uint (pull_data->progress, "scanned-metadata", n_scanned_metadata);
+  ostree_async_progress_set_uint64 (pull_data->progress, "bytes-transferred", bytes_transferred);
 
   if (pull_data->fetching_sync_uri)
     {
       gs_free char *uri_string = soup_uri_to_string (pull_data->fetching_sync_uri, TRUE);
-      g_string_append_printf (status, "Requesting %s", uri_string);
+      gs_free char *status_string = g_strconcat ("Requesting ", uri_string, NULL);
+      ostree_async_progress_set_status (pull_data->progress, status_string);
     }
-  else if (outstanding_fetches)
-    {
-      guint64 bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
-      guint fetched = pull_data->n_fetched_metadata + pull_data->n_fetched_content;
-      guint requested = pull_data->n_requested_metadata + pull_data->n_requested_content;
-      gs_free char *formatted_bytes_transferred = NULL;
-
-      formatted_bytes_transferred = g_format_size_full (bytes_transferred, 0);
-
-      g_string_append_printf (status, "Receiving objects: %u%% (%u/%u) %s",
-                              (guint)((((double)fetched) / requested) * 100),
-                              fetched, requested, formatted_bytes_transferred);
-    }
-  else if (outstanding_writes > 0)
-    g_string_append_printf (status, "Writing objects: %u", outstanding_writes);
-  else if (!pull_data->metadata_scan_idle)
-    g_string_append_printf (status, "Scanning metadata: %u",
-                            g_atomic_int_get (&pull_data->n_scanned_metadata));
   else
-    g_string_append_printf (status, "Idle");
-
-  gs_console_begin_status_line (gs_console_get (), status->str, NULL, NULL);
-
-  g_string_free (status, TRUE);
+    ostree_async_progress_set_status (pull_data->progress, NULL);
 
   return TRUE;
 }
@@ -231,8 +221,7 @@ pull_worker_message_new (int msgtype, gpointer data)
   msg->t = msgtype;
   switch (msgtype)
     {
-    case PULL_MSG_SCAN_IDLE:
-    case PULL_MSG_MAIN_IDLE:
+    case PULL_MSG_IDLE:
       msg->d.idle_serial = GPOINTER_TO_UINT (data);
       break;
     case PULL_MSG_SCAN:
@@ -273,9 +262,7 @@ check_outstanding_requests_handle_error (OtPullData          *pull_data,
                                  pull_data->n_outstanding_content_fetches == 0);
   gboolean current_write_idle = (pull_data->n_outstanding_metadata_write_requests == 0 &&
                                  pull_data->n_outstanding_content_write_requests == 0);
-
-  g_debug ("pull: scan: %u fetching: %u staging: %u",
-           !pull_data->metadata_scan_idle, !current_fetch_idle, !current_write_idle);
+  gboolean current_idle = current_fetch_idle && current_write_idle;
 
   throw_async_error (pull_data, error);
 
@@ -286,8 +273,26 @@ check_outstanding_requests_handle_error (OtPullData          *pull_data,
         g_main_loop_quit (pull_data->loop);
       return;
     }
-  else if (pull_data->metadata_scan_idle && current_fetch_idle && current_write_idle)
+
+  if (pull_data->was_idle && !current_idle)
     {
+      /* We transitioned to !idle */
+      g_debug ("pull: No longer idle");
+      pull_data->idle_serial++;
+      pull_data->was_idle = FALSE;
+    }
+  else if (!pull_data->was_idle && current_idle)
+    {
+      pull_data->was_idle = TRUE;
+      g_debug ("Sending new MSG_IDLE with serial %u", pull_data->idle_serial);
+      ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
+                              pull_worker_message_new (PULL_MSG_IDLE, GUINT_TO_POINTER (pull_data->idle_serial)));
+    }
+
+  if (pull_data->metadata_scan_complete && current_idle)
+    {
+      g_debug ("pull: metadata scan complete and idle, exiting mainloop");
+
       g_main_loop_quit (pull_data->loop);
     }
 }
@@ -303,31 +308,23 @@ static gboolean
 run_mainloop_monitor_fetcher (OtPullData   *pull_data)
 {
   GSource *update_timeout = NULL;
-  GSConsole *console;
   GSource *idle_src;
 
-  console = gs_console_get ();
-
-  if (console)
+  if (pull_data->progress)
     {
-      gs_console_begin_status_line (console, "", NULL, NULL);
-
       update_timeout = g_timeout_source_new_seconds (1);
-      g_source_set_callback (update_timeout, uri_fetch_update_status, pull_data, NULL);
+      g_source_set_callback (update_timeout, update_progress, pull_data, NULL);
       g_source_attach (update_timeout, g_main_loop_get_context (pull_data->loop));
       g_source_unref (update_timeout);
     }
-  
+
   idle_src = g_idle_source_new ();
   g_source_set_callback (idle_src, idle_check_outstanding_requests, pull_data, NULL);
   g_source_attach (idle_src, pull_data->main_context);
   g_main_loop_run (pull_data->loop);
 
-  if (console)
-    {
-      gs_console_end_status_line (console, NULL, NULL);
-      g_source_destroy (update_timeout);
-    }
+  if (update_timeout)
+    g_source_destroy (update_timeout);
 
   return !pull_data->caught_error;
 }
@@ -439,7 +436,7 @@ scan_dirtree_object (OtPullData   *pull_data,
       const char *filename;
       gboolean file_is_stored;
       gs_unref_variant GVariant *csum = NULL;
-      gs_free char *file_checksum;
+      gs_free char *file_checksum = NULL;
 
       g_variant_get_child (files_variant, i, "(&s@ay)", &filename, &csum);
 
@@ -456,6 +453,7 @@ scan_dirtree_object (OtPullData   *pull_data,
         {
           g_hash_table_insert (pull_data->requested_content, file_checksum, file_checksum);
       
+          g_debug ("queued fetch of content %s", file_checksum);
           ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
                                   pull_worker_message_new (PULL_MSG_FETCH,
                                                            ostree_object_name_serialize (file_checksum, OSTREE_OBJECT_TYPE_FILE)));
@@ -612,6 +610,14 @@ content_fetch_on_complete (GObject        *object,
   check_outstanding_requests_handle_error (pull_data, local_error);
 }
 
+static void
+note_metadata_not_complete (OtPullData *pull_data)
+{
+  if (pull_data->metadata_scan_complete)
+    g_debug ("pull: Transition metadata scan complete -> not complete");
+  pull_data->metadata_scan_complete = FALSE;
+}
+
 static void
 on_metadata_writed (GObject           *object,
                     GAsyncResult      *result,
@@ -625,6 +631,7 @@ on_metadata_writed (GObject           *object,
   OstreeObjectType objtype;
   gs_free char *checksum = NULL;
   gs_free guchar *csum = NULL;
+  gs_free char *stringified_object = NULL;
 
   if (!ostree_repo_write_metadata_finish ((OstreeRepo*)object, result, 
                                           &csum, error))
@@ -635,7 +642,8 @@ on_metadata_writed (GObject           *object,
   ostree_object_name_deserialize (fetch_data->object, &expected_checksum, &objtype);
   g_assert (OSTREE_OBJECT_TYPE_IS_META (objtype));
 
-  g_debug ("write of %s complete", ostree_object_to_string (checksum, objtype));
+  stringified_object = ostree_object_to_string (checksum, objtype);
+  g_debug ("write of %s complete", stringified_object);
 
   if (strcmp (checksum, expected_checksum) != 0)
     {
@@ -645,7 +653,7 @@ on_metadata_writed (GObject           *object,
       goto out;
     }
 
-  pull_data->metadata_scan_idle = FALSE;
+  note_metadata_not_complete (pull_data);
   ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
                           pull_worker_message_new (PULL_MSG_SCAN,
                                                   g_variant_ref (fetch_data->object)));
@@ -912,7 +920,7 @@ on_metadata_objects_to_scan_ready (gint         fd,
           g_variant_unref (msg->d.item);
           g_free (msg);
         }
-      else if (msg->t == PULL_MSG_MAIN_IDLE)
+      else if (msg->t == PULL_MSG_IDLE)
         {
           g_free (last_idle_msg);
           last_idle_msg = msg;
@@ -920,6 +928,7 @@ on_metadata_objects_to_scan_ready (gint         fd,
       else if (msg->t == PULL_MSG_QUIT)
         {
           g_free (msg);
+          g_debug ("pull: Processing PULL_MSG_QUIT");
           g_main_loop_quit (pull_data->metadata_thread_loop);
         }
       else
@@ -927,13 +936,12 @@ on_metadata_objects_to_scan_ready (gint         fd,
       }
     
   if (last_idle_msg)
-    ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
-                            last_idle_msg);
+    {
+      g_debug ("pull: Processing PULL_MSG_IDLE");
+      ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
+                              last_idle_msg);
+    }
   
-  /* When we have no queue to process, notify the main thread */
-  ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
-                          pull_worker_message_new (PULL_MSG_SCAN_IDLE, GUINT_TO_POINTER (0)));
-
  out:
   if (local_error)
     {
@@ -1029,29 +1037,21 @@ on_metadata_objects_to_fetch_ready (gint         fd,
   if (!ot_waitable_queue_pop (pull_data->metadata_objects_to_fetch, (gpointer*)&msg))
     goto out;
 
-  if (msg->t == PULL_MSG_MAIN_IDLE)
+  if (msg->t == PULL_MSG_IDLE)
     {
+      pull_data->checking_metadata_scan_complete = FALSE;
       if (msg->d.idle_serial == pull_data->idle_serial)
         {
-          g_assert (!pull_data->metadata_scan_idle);
-          pull_data->metadata_scan_idle = TRUE;
-          g_debug ("pull: metadata scan is idle");
-        }
-    }
-  else if (msg->t == PULL_MSG_SCAN_IDLE)
-    {
-      if (!pull_data->metadata_scan_idle)
-        {
-          g_debug ("pull: queue MAIN_IDLE");
-          pull_data->idle_serial++;
-          ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
-                                  pull_worker_message_new (PULL_MSG_MAIN_IDLE, GUINT_TO_POINTER (pull_data->idle_serial)));
+          g_debug ("marking metadata scan as complete");
+          pull_data->metadata_scan_complete = TRUE;
         }
     }
   else if (msg->t == PULL_MSG_FETCH || msg->t == PULL_MSG_FETCH_DETACHED_METADATA)
     {
       gboolean is_detached_meta;
 
+      note_metadata_not_complete (pull_data);
+
       is_detached_meta = msg->t == PULL_MSG_FETCH_DETACHED_METADATA;
       
       enqueue_one_object_request (pull_data, msg->d.item, is_detached_meta);
@@ -1198,6 +1198,7 @@ ostree_repo_pull (OstreeRepo               *self,
                   const char               *remote_name,
                   char                    **refs_to_fetch,
                   OstreeRepoPullFlags       flags,
+                  OstreeAsyncProgress      *progress,
                   GCancellable             *cancellable,
                   GError                  **error)
 {
@@ -1231,6 +1232,7 @@ ostree_repo_pull (OstreeRepo               *self,
   pull_data->flags = flags;
 
   pull_data->repo = self;
+  pull_data->progress = progress;
 
   pull_data->scanned_metadata = g_hash_table_new_full (ostree_hash_object_name, g_variant_equal,
                                                        (GDestroyNotify)g_variant_unref, NULL);
@@ -1376,6 +1378,8 @@ ostree_repo_pull (OstreeRepo               *self,
                                         cancellable, error))
     goto out;
 
+  g_debug ("resuming transaction: %s", pull_data->transaction_resuming ? "true" : " false");
+
   pull_data->metadata_objects_to_fetch = ot_waitable_queue_new ();
   pull_data->metadata_objects_to_scan = ot_waitable_queue_new ();
   pull_data->metadata_thread = g_thread_new ("metadatascan", metadata_thread_main, pull_data);
@@ -1409,15 +1413,14 @@ ostree_repo_pull (OstreeRepo               *self,
     g_source_unref (queue_src);
   }
 
-  /* Prime the message queue */
-  pull_data->idle_serial++;
-  ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
-                          pull_worker_message_new (PULL_MSG_MAIN_IDLE, GUINT_TO_POINTER (pull_data->idle_serial)));
-  
   /* Now await work completion */
   if (!run_mainloop_monitor_fetcher (pull_data))
     goto out;
   
+  g_assert_cmpint (pull_data->n_outstanding_metadata_fetches, ==, 0);
+  g_assert_cmpint (pull_data->n_outstanding_metadata_write_requests, ==, 0);
+  g_assert_cmpint (pull_data->n_outstanding_content_fetches, ==, 0);
+  g_assert_cmpint (pull_data->n_outstanding_content_write_requests, ==, 0);
 
   g_hash_table_iter_init (&hash_iter, updated_refs);
   while (g_hash_table_iter_next (&hash_iter, &key, &value))
