@@ -116,6 +116,7 @@ ostree_repo_finalize (GObject *object)
   g_clear_pointer (&self->txn_refs, g_hash_table_destroy);
   g_clear_pointer (&self->cached_meta_indexes, (GDestroyNotify) g_ptr_array_unref);
   g_clear_pointer (&self->cached_content_indexes, (GDestroyNotify) g_ptr_array_unref);
+  g_clear_pointer (&self->object_sizes, (GDestroyNotify) g_hash_table_unref);
   g_mutex_clear (&self->cache_lock);
   g_mutex_clear (&self->txn_stats_lock);
 
@@ -1038,8 +1039,8 @@ ostree_repo_load_file (OstreeRepo         *self,
               gs_unref_object GFile *full_path =
                     _ostree_repo_get_object_path (self, checksum, OSTREE_OBJECT_TYPE_FILE);
 
-              if (!ostree_get_xattrs_for_file (full_path, &ret_xattrs,
-                                               cancellable, error))
+              if (!gs_file_get_all_xattrs (full_path, &ret_xattrs,
+                                           cancellable, error))
                 goto out;
             }
 
@@ -1332,7 +1333,7 @@ ostree_repo_load_variant_if_exists (OstreeRepo       *self,
  * @self: Repo
  * @objtype: Expected object type
  * @sha256: Checksum string
- * @out_variant: (out): (transfer full): Metadata object
+ * @out_variant: (out) (transfer full): Metadata object
  * @error: Error
  *
  * Load the metadata object @sha256 of type @objtype, storing the
@@ -1453,6 +1454,7 @@ ostree_repo_read_commit (OstreeRepo   *self,
  * @remote_name: Name of remote
  * @refs_to_fetch: (array zero-terminated=1) (element-type utf8) (allow-none): Optional list of refs; if %NULL, fetch all configured refs
  * @flags: Options controlling fetch behavior
+ * @progress: (allow-none): Progress
  * @cancellable: Cancellable
  * @error: Error
  *
@@ -1466,6 +1468,7 @@ ostree_repo_pull (OstreeRepo               *self,
                   const char               *remote_name,
                   char                    **refs_to_fetch,
                   OstreeRepoPullFlags       flags,
+                  OstreeAsyncProgress      *progress,
                   GCancellable             *cancellable,
                   GError                  **error)
 {
@@ -1475,7 +1478,17 @@ ostree_repo_pull (OstreeRepo               *self,
 }
 #endif
 
-#ifdef HAVE_GPGME
+/**
+ * ostree_repo_sign_commit:
+ * @self: Self
+ * @commit_checksum: SHA256 of given commit to sign
+ * @key_id: Use this GPG key id
+ * @homedir: (allow-none): GPG home directory, or %NULL
+ * @cancellable: A #GCancellable
+ * @error: a #GError
+ *
+ * Add a GPG signature to a commit.
+ */
 gboolean
 ostree_repo_sign_commit (OstreeRepo     *self,
                          const gchar    *commit_checksum,
@@ -1484,6 +1497,7 @@ ostree_repo_sign_commit (OstreeRepo     *self,
                          GCancellable   *cancellable,
                          GError        **error)
 {
+#ifdef HAVE_GPGME
   gboolean ret = FALSE;
   gs_unref_object GFile *commit_path = NULL;
   gs_unref_variant GVariant *metadata = NULL;
@@ -1645,6 +1659,11 @@ out:
   if (signature_file)
     g_mapped_file_unref (signature_file);
   return ret;
+#else
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+               "This version of ostree was compiled without GPG support");
+  return FALSE;
+#endif
 }
 
 /**
@@ -1667,6 +1686,7 @@ ostree_repo_verify_commit (OstreeRepo   *self,
                            GCancellable *cancellable,
                            GError      **error)
 {
+#ifdef HAVE_GPGME
   gboolean ret = FALSE;
   gs_unref_object OstreeGpgVerifier *verifier = NULL;
   gs_unref_variant GVariant *commit_variant = NULL;
@@ -1715,7 +1735,7 @@ ostree_repo_verify_commit (OstreeRepo   *self,
   if (!signaturedata)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                   "No signatures found");
+                   "GPG verification enabled, but no signatures found (use gpg-verify=false in remote config to disable)");
       goto out;
     }
 
@@ -1775,6 +1795,9 @@ out:
   if (commit_tmp_path)
     (void) gs_file_unlink (commit_tmp_path, NULL, NULL);
   return ret;
-}
-
+#else
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+               "This version of ostree was compiled without GPG support");
+  return FALSE;
 #endif
+}
