@@ -24,6 +24,7 @@
 
 #include "ostree-core.h"
 #include "ostree-types.h"
+#include "ostree-async-progress.h"
 
 G_BEGIN_DECLS
 
@@ -281,7 +282,8 @@ typedef OstreeRepoCommitFilterResult (*OstreeRepoCommitFilter) (OstreeRepo    *r
  */
 typedef enum {
   OSTREE_REPO_COMMIT_MODIFIER_FLAGS_NONE = 0,
-  OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS = (1 << 0)
+  OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS = (1 << 0),
+  OSTREE_REPO_COMMIT_MODIFIER_FLAGS_GENERATE_SIZES = (1 << 1)
 } OstreeRepoCommitModifierFlags;
 
 /**
@@ -460,10 +462,10 @@ gboolean ostree_repo_pull (OstreeRepo             *self,
                            const char             *remote_name,
                            char                  **refs_to_fetch,
                            OstreeRepoPullFlags     flags,
+                           OstreeAsyncProgress    *progress,
                            GCancellable           *cancellable,
                            GError                **error);
 
-#ifdef HAVE_GPGME
 gboolean ostree_repo_sign_commit (OstreeRepo     *self,
                                   const gchar    *commit_checksum,
                                   const gchar    *key_id,
@@ -477,7 +479,6 @@ gboolean ostree_repo_verify_commit (OstreeRepo   *self,
                                     GFile        *extra_keyring,
                                     GCancellable *cancellable,
                                     GError      **error);
-#endif
 
 G_END_DECLS
 
