@@ -23,27 +23,33 @@
 #include "config.h"
 
 #include "ostree-sysroot-private.h"
+#include "ostree-core-private.h"
 #include "otutil.h"
 #include "libgsystem.h"
 
 /**
- * copy_one_config_file:
+ * copy_modified_config_file:
  *
  * Copy @file from @modified_etc to @new_etc, overwriting any existing
- * file there.
+ * file there.  The @file may refer to a regular file, a symbolic
+ * link, or a directory.  Directories will be copied recursively.
+ *
+ * Note this function does not (yet) handle the case where a directory
+ * needed by a modified file is deleted in a newer tree.
  */
 static gboolean
-copy_one_config_file (GFile              *orig_etc,
-                      GFile              *modified_etc,
-                      GFile              *new_etc,
-                      GFile              *src,
-                      GCancellable       *cancellable,
-                      GError            **error)
+copy_modified_config_file (GFile              *orig_etc,
+                           GFile              *modified_etc,
+                           GFile              *new_etc,
+                           GFile              *src,
+                           GCancellable       *cancellable,
+                           GError            **error)
 {
   gboolean ret = FALSE;
   gs_unref_object GFileInfo *src_info = NULL;
+  gs_unref_object GFileInfo *parent_info = NULL;
   gs_unref_object GFile *dest = NULL;
-  gs_unref_object GFile *parent = NULL;
+  gs_unref_object GFile *dest_parent = NULL;
   gs_free char *relative_path = NULL;
   
   relative_path = g_file_get_relative_path (modified_etc, src);
@@ -55,49 +61,28 @@ copy_one_config_file (GFile              *orig_etc,
   if (!src_info)
     goto out;
 
+  dest_parent = g_file_get_parent (dest);
+  if (!ot_gfile_query_info_allow_noent (dest_parent, OSTREE_GIO_FAST_QUERYINFO, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                        &parent_info, cancellable, error))
+    goto out;
+  if (!parent_info)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "New tree removes parent directory '%s', cannot merge",
+                   gs_file_get_path_cached (dest_parent));
+      goto out;
+    }
+
+  if (!gs_shutil_rm_rf (dest, cancellable, error))
+    goto out;
+
   if (g_file_info_get_file_type (src_info) == G_FILE_TYPE_DIRECTORY)
     {
-      gs_unref_object GFileEnumerator *src_enum = NULL;
-      gs_unref_object GFileInfo *child_info = NULL;
-      GError *temp_error = NULL;
-
-      /* FIXME actually we need to copy permissions and xattrs */
-      if (!gs_file_ensure_directory (dest, TRUE, cancellable, error))
+      if (!gs_shutil_cp_a (src, dest, cancellable, error))
         goto out;
-
-      src_enum = g_file_enumerate_children (src, OSTREE_GIO_FAST_QUERYINFO,
-                                            G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                            cancellable, error);
-
-      while ((child_info = g_file_enumerator_next_file (src_enum, cancellable, error)) != NULL)
-        {
-          gs_unref_object GFile *child = g_file_get_child (src, g_file_info_get_name (child_info));
-
-          if (!copy_one_config_file (orig_etc, modified_etc, new_etc, child,
-                                     cancellable, error))
-            goto out;
-        }
-      g_clear_object (&child_info);
-      if (temp_error != NULL)
-        {
-          g_propagate_error (error, temp_error);
-          goto out;
-        }
     }
   else
     {
-      parent = g_file_get_parent (dest);
-
-      /* FIXME actually we need to copy permissions and xattrs */
-      if (!gs_file_ensure_directory (parent, TRUE, cancellable, error))
-        goto out;
-      
-      /* We unlink here because otherwise gio throws an error on
-       * dangling symlinks.
-       */
-      if (!ot_gfile_ensure_unlinked (dest, cancellable, error))
-        goto out;
-
       if (!g_file_copy (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_ALL_METADATA,
                         cancellable, NULL, NULL, error))
         goto out;
@@ -169,16 +154,16 @@ merge_etc_changes (GFile          *orig_etc,
     {
       OstreeDiffItem *diff = modified->pdata[i];
 
-      if (!copy_one_config_file (orig_etc, modified_etc, new_etc, diff->target,
-                                 cancellable, error))
+      if (!copy_modified_config_file (orig_etc, modified_etc, new_etc, diff->target,
+                                      cancellable, error))
         goto out;
     }
   for (i = 0; i < added->len; i++)
     {
       GFile *file = added->pdata[i];
 
-      if (!copy_one_config_file (orig_etc, modified_etc, new_etc, file,
-                                 cancellable, error))
+      if (!copy_modified_config_file (orig_etc, modified_etc, new_etc, file,
+                                      cancellable, error))
         goto out;
     }
 
@@ -616,7 +601,7 @@ install_deployment_kernel (OstreeSysroot   *sysroot,
   gs_free char *version_key = NULL;
   gs_free char *ostree_kernel_arg = NULL;
   gs_free char *options_key = NULL;
-  __attribute__((cleanup(_ostree_ordered_hash_cleanup))) OstreeOrderedHash *ohash = NULL;
+  __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *kargs = NULL;
   const char *val;
   OstreeBootconfigParser *bootconfig;
   gsize len;
@@ -708,12 +693,14 @@ install_deployment_kernel (OstreeSysroot   *sysroot,
     }
 
   val = ostree_bootconfig_parser_get (bootconfig, "options");
-  ostree_kernel_arg = g_strdup_printf ("/ostree/boot.%d/%s/%s/%d",
+
+  ostree_kernel_arg = g_strdup_printf ("ostree=/ostree/boot.%d/%s/%s/%d",
                                        new_bootversion, osname, bootcsum,
                                        ostree_deployment_get_bootserial (deployment));
-  ohash = _ostree_sysroot_parse_kernel_args (val);
-  _ostree_ordered_hash_replace_key (ohash, "ostree", ostree_kernel_arg);
-  options_key = _ostree_sysroot_kernel_arg_string_serialize (ohash);
+  kargs = _ostree_kernel_args_from_string (val);
+  _ostree_kernel_args_replace_take (kargs, ostree_kernel_arg);
+  ostree_kernel_arg = NULL;
+  options_key = _ostree_kernel_args_to_string (kargs);
   ostree_bootconfig_parser_set (bootconfig, "options", options_key);
   
   if (!ostree_bootconfig_parser_write (ostree_deployment_get_bootconfig (deployment), bootconfpath,
@@ -752,8 +739,7 @@ swap_bootloader (OstreeSysroot  *sysroot,
 }
 
 static GHashTable *
-bootcsum_counts_for_deployment_list (GPtrArray   *deployments,
-                                     gboolean     set_bootserial)
+assign_bootserials (GPtrArray   *deployments)
 {
   guint i;
   GHashTable *ret = 
@@ -766,10 +752,47 @@ bootcsum_counts_for_deployment_list (GPtrArray   *deployments,
       guint count;
 
       count = GPOINTER_TO_UINT (g_hash_table_lookup (ret, bootcsum));
-      g_hash_table_replace (ret, (char*)bootcsum, GUINT_TO_POINTER (count + 1));
+      g_hash_table_replace (ret, (char*) bootcsum,
+                            GUINT_TO_POINTER (count + 1));
 
-      if (set_bootserial)
-        ostree_deployment_set_bootserial (deployment, count);
+      ostree_deployment_set_bootserial (deployment, count);
+    }
+  return ret;
+}
+
+static GHashTable *
+bootconfig_counts_for_deployment_list (GPtrArray   *deployments)
+{
+  guint i;
+  GHashTable *ret = 
+    g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+
+  for (i = 0; i < deployments->len; i++)
+    {
+      OstreeDeployment *deployment = deployments->pdata[i];
+      const char *bootcsum = ostree_deployment_get_bootcsum (deployment);
+      OstreeBootconfigParser *bootconfig = ostree_deployment_get_bootconfig (deployment);
+      const char *boot_options = ostree_bootconfig_parser_get (bootconfig, "options");
+      GChecksum *bootconfig_checksum = g_checksum_new (G_CHECKSUM_SHA256);
+      const char *bootconfig_checksum_str;
+      __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *kargs = NULL;
+      gs_free char *boot_options_without_ostree = NULL;
+      guint count;
+      
+      /* We checksum the kernel arguments *except* ostree= */
+      kargs = _ostree_kernel_args_from_string (boot_options);
+      _ostree_kernel_args_replace (kargs, "ostree");
+      boot_options_without_ostree = _ostree_kernel_args_to_string (kargs);
+
+      g_checksum_update (bootconfig_checksum, (guint8*)bootcsum, strlen (bootcsum));
+      g_checksum_update (bootconfig_checksum, (guint8*)boot_options_without_ostree,
+                         strlen (boot_options_without_ostree));
+
+      bootconfig_checksum_str = g_checksum_get_string (bootconfig_checksum);
+
+      count = GPOINTER_TO_UINT (g_hash_table_lookup (ret, bootconfig_checksum_str));
+      g_hash_table_replace (ret, g_strdup (bootconfig_checksum_str),
+                            GUINT_TO_POINTER (count + 1));
     }
   return ret;
 }
@@ -836,19 +859,17 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
   guint i;
   gboolean requires_new_bootversion = FALSE;
   gboolean found_booted_deployment = FALSE;
-  gs_unref_hashtable GHashTable *new_bootcsum_to_count = NULL;
 
   g_assert (self->loaded);
 
-  /* Calculate the total number of deployments per bootcsums; while we
-   * are doing this, assign a bootserial to each new deployment.
+  /* Assign a bootserial to each new deployment.
    */
-  new_bootcsum_to_count = bootcsum_counts_for_deployment_list (new_deployments, TRUE);
+  assign_bootserials (new_deployments);
 
   /* Determine whether or not we need to touch the bootloader
-   * configuration.  If we have an equal number of deployments and
-   * more strongly an equal number of deployments per bootcsum, then
-   * we can just swap the subbootversion bootlinks.
+   * configuration.  If we have an equal number of deployments with
+   * matching bootloader configuration, then we can just swap the
+   * subbootversion bootlinks.
    */
   if (new_deployments->len != self->deployments->len)
     requires_new_bootversion = TRUE;
@@ -856,14 +877,16 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
     {
       GHashTableIter hashiter;
       gpointer hkey, hvalue;
-      gs_unref_hashtable GHashTable *orig_bootcsum_to_count
-        = bootcsum_counts_for_deployment_list (self->deployments, FALSE);
+      gs_unref_hashtable GHashTable *new_bootconfig_to_count = 
+        bootconfig_counts_for_deployment_list (new_deployments);
+      gs_unref_hashtable GHashTable *orig_bootconfig_to_count
+        = bootconfig_counts_for_deployment_list (self->deployments);
 
-      g_hash_table_iter_init (&hashiter, orig_bootcsum_to_count);
+      g_hash_table_iter_init (&hashiter, orig_bootconfig_to_count);
       while (g_hash_table_iter_next (&hashiter, &hkey, &hvalue))
         {
           guint orig_count = GPOINTER_TO_UINT (hvalue);
-          gpointer new_countp = g_hash_table_lookup (new_bootcsum_to_count, hkey);
+          gpointer new_countp = g_hash_table_lookup (new_bootconfig_to_count, hkey);
           guint new_count = GPOINTER_TO_UINT (new_countp);
 
           if (orig_count != new_count)
@@ -1028,13 +1051,13 @@ allocate_deployserial (OstreeSysroot           *self,
 }
                             
 /**
- * ostree_sysroot_deploy_one_tree:
+ * ostree_sysroot_deploy_tree:
  * @self: Sysroot
  * @osname: (allow-none): osname to use for merge deployment
  * @revision: Checksum to add
  * @origin: (allow-none): Origin to use for upgrades
- * @add_kernel_argv: (allow-none): Append these arguments to kernel configuration
  * @provided_merge_deployment: (allow-none): Use this deployment for merge path
+ * @override_kernel_argv: (allow-none) (array zero-terminated=1) (element-type utf8): Use these as kernel arguments; if %NULL, inherit options from provided_merge_deployment
  * @out_new_deployment: (out): The new deployment path
  * @cancellable: Cancellable
  * @error: Error
@@ -1043,21 +1066,22 @@ allocate_deployserial (OstreeSysroot           *self,
  * way merge with @provided_merge_deployment for configuration.
  */
 gboolean
-ostree_sysroot_deploy_one_tree (OstreeSysroot     *self,
-                                const char        *osname,
-                                const char        *revision,
-                                GKeyFile          *origin,
-                                char             **add_kernel_argv,
-                                OstreeDeployment  *provided_merge_deployment,
-                                OstreeDeployment **out_new_deployment,
-                                GCancellable      *cancellable,
-                                GError           **error)
+ostree_sysroot_deploy_tree (OstreeSysroot     *self,
+                            const char        *osname,
+                            const char        *revision,
+                            GKeyFile          *origin,
+                            OstreeDeployment  *provided_merge_deployment,
+                            char             **override_kernel_argv,
+                            OstreeDeployment **out_new_deployment,
+                            GCancellable      *cancellable,
+                            GError           **error)
 {
   gboolean ret = FALSE;
   gint new_deployserial;
   gs_unref_object OstreeDeployment *new_deployment = NULL;
   gs_unref_object OstreeDeployment *merge_deployment = NULL;
   gs_unref_object OstreeRepo *repo = NULL;
+  gs_unref_object GFile *osdeploydir = NULL;
   gs_unref_object GFile *commit_root = NULL;
   gs_unref_object GFile *tree_kernel_path = NULL;
   gs_unref_object GFile *tree_initramfs_path = NULL;
@@ -1070,6 +1094,14 @@ ostree_sysroot_deploy_one_tree (OstreeSysroot     *self,
   if (osname == NULL)
     osname = ostree_deployment_get_osname (self->booted_deployment);
 
+  osdeploydir = ot_gfile_get_child_build_path (self->path, "ostree", "deploy", osname, NULL);
+  if (!g_file_query_exists (osdeploydir, NULL))
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "No OS named \"%s\" known", osname);
+      goto out;
+    }
+
   if (!ostree_sysroot_get_repo (self, &repo, cancellable, error))
     goto out;
 
@@ -1130,30 +1162,17 @@ ostree_sysroot_deploy_one_tree (OstreeSysroot     *self,
       goto out;
     }
 
-  /* We have inherited kernel arguments from the previous deployment;
-   * now, override/extend that with arguments provided by the command
-   * line.
-   * 
-   * After this, install_deployment_kernel() will set the other boot
+  /* After this, install_deployment_kernel() will set the other boot
    * options and write it out to disk.
    */
-  if (add_kernel_argv)
+  if (override_kernel_argv)
     {
-      char **strviter;
-      __attribute__((cleanup(_ostree_ordered_hash_cleanup))) OstreeOrderedHash *ohash = NULL;
+      __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *kargs = NULL;
       gs_free char *new_options = NULL;
 
-      ohash = _ostree_sysroot_parse_kernel_args (ostree_bootconfig_parser_get (bootconfig, "options"));
-
-      for (strviter = add_kernel_argv; *strviter; strviter++)
-        {
-          char *karg = g_strdup (*strviter);
-          const char *val = _ostree_sysroot_split_keyeq (karg);
-        
-          _ostree_ordered_hash_replace_key_take (ohash, karg, val);
-        }
-
-      new_options = _ostree_sysroot_kernel_arg_string_serialize (ohash);
+      kargs = _ostree_kernel_args_new ();
+      _ostree_kernel_args_append_argv (kargs, override_kernel_argv);
+      new_options = _ostree_kernel_args_to_string (kargs);
       ostree_bootconfig_parser_set (bootconfig, "options", new_options);
     }
 
