@@ -24,6 +24,7 @@
 
 #include "ot-admin-builtins.h"
 #include "ot-admin-functions.h"
+#include "ot-builtins-common.h"
 #include "ostree.h"
 #include "otutil.h"
 #include "libgsystem.h"
@@ -66,50 +67,33 @@ ot_admin_builtin_upgrade (int argc, char **argv, OstreeSysroot *sysroot, GCancel
   if (!ostree_sysroot_load (sysroot, cancellable, error))
     goto out;
 
-  if (!ot_admin_require_booted_deployment_or_osname (sysroot, opt_osname,
-                                                     cancellable, error))
-    goto out;
-  if (!opt_osname)
-    opt_osname = (char*)ostree_deployment_get_osname (ostree_sysroot_get_booted_deployment (sysroot));
-  merge_deployment = ostree_sysroot_get_merge_deployment (sysroot, opt_osname); 
-  if (merge_deployment == NULL)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "No previous deployment for OS '%s'", opt_osname);
-      goto out;
-    }
-
-  deployment_path = ostree_sysroot_get_deployment_directory (sysroot, merge_deployment);
-  deployment_origin_path = ostree_sysroot_get_deployment_origin_path (deployment_path);
 
   if (!ostree_sysroot_get_repo (sysroot, &repo, cancellable, error))
     goto out;
 
-  origin = ostree_deployment_get_origin (merge_deployment);
-  if (!origin)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "No origin known for current deployment");
-      goto out;
-    }
-  origin_refspec = g_key_file_get_string (origin, "origin", "refspec", NULL);
-  if (!origin_refspec)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "No origin/refspec in current deployment origin; cannot upgrade via ostree");
-      goto out;
-    }
-  if (!ostree_parse_refspec (origin_refspec, &origin_remote, &origin_ref, error))
+  if (!ot_admin_deploy_prepare (sysroot, opt_osname, &merge_deployment,
+                                &origin_remote, &origin_ref,
+                                &origin,
+                                cancellable, error))
     goto out;
 
   if (origin_remote)
     {
       OstreeRepoPullFlags pullflags = 0;
       char *refs_to_fetch[] = { origin_ref, NULL };
+      GSConsole *console;
+      gs_unref_object OstreeAsyncProgress *progress = NULL;
+
+      console = gs_console_get ();
+      if (console)
+        {
+          gs_console_begin_status_line (console, "", NULL, NULL);
+          progress = ostree_async_progress_new_and_connect (ot_common_pull_progress, console);
+        }
 
       g_print ("Fetching remote %s ref %s\n", origin_remote, origin_ref);
 
-      if (!ostree_repo_pull (repo, origin_remote, refs_to_fetch, pullflags,
+      if (!ostree_repo_pull (repo, origin_remote, refs_to_fetch, pullflags, progress,
                              cancellable, error))
         goto out;
     }
@@ -139,12 +123,12 @@ ot_admin_builtin_upgrade (int argc, char **argv, OstreeSysroot *sysroot, GCancel
           goto out;
         }
 
-      if (!ostree_sysroot_deploy_one_tree (sysroot,
-                                           opt_osname, new_revision, origin,
-                                           NULL,
-                                           merge_deployment,
-                                           &new_deployment,
-                                           cancellable, error))
+      if (!ostree_sysroot_deploy_tree (sysroot,
+                                       opt_osname, new_revision, origin,
+                                       merge_deployment,
+                                       NULL,
+                                       &new_deployment,
+                                       cancellable, error))
         goto out;
 
       if (!ot_admin_complete_deploy_one (sysroot, opt_osname,
