@@ -52,14 +52,13 @@ split_key_string (const char   *k,
 }
 
 gboolean
-ostree_builtin_config (int argc, char **argv, GFile *repo_path, GCancellable *cancellable, GError **error)
+ostree_builtin_config (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
 {
   GOptionContext *context = NULL;
   gboolean ret = FALSE;
   const char *op;
   const char *section_key;
   const char *value;
-  gs_unref_object OstreeRepo *repo = NULL;
   gs_free char *section = NULL;
   gs_free char *key = NULL;
   GKeyFile *config = NULL;
@@ -70,10 +69,6 @@ ostree_builtin_config (int argc, char **argv, GFile *repo_path, GCancellable *ca
   if (!g_option_context_parse (context, &argc, &argv, error))
     goto out;
 
-  repo = ostree_repo_new (repo_path);
-  if (!ostree_repo_check (repo, error))
-    goto out;
-
   if (argc < 2)
     {
       ot_util_usage_error (context, "OPERATION must be specified", error);
